//-*-c++-*-
//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2017 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
// Purpose        : Abstract interface for general "vector loading"
//                  callback classes.
//                  
//
// Special Notes  : This can be used to implement a generic call-back interface
//                  for external simulators.  The GeneralExternal device
//                  will use this to call back to the external simulator to
//                  get the DAE vector and matrix elements associated with
//                  the device.
//
// Creator        : Tom Russo, SNL, Electrical Models and Simulation
//
// Creation Date  : 2/28/2017
//
//
//
//
//-----------------------------------------------------------------------------

#ifndef N_DEV_VectorComputeInterface_H
#define N_DEV_VectorComputeInterface_H
#include <string>
#include <vector>
namespace Xyce {
namespace Device {

/// Abstract interface class for allowing the GeneralExternal device
/// to obtain DAE vectors and matrices (e.g. from an external simulator
/// or otherwise)
class VectorComputeInterface
{
public:
  virtual bool computeXyceVectors(std::vector<double> & solutionVars,
                                  double time,
                                  std::vector<double>&F,
                                  std::vector<double>&Q,
                                  std::vector<double>&B,
                                  std::vector<std::vector<double> > &dFdx,
                                  std::vector<std::vector<double> > &dQdx) = 0;
};
   
} // namespace Device
} // namespace Xyce
#endif
