//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-Transient04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2017 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  :
//
// Creator        : Dave Baur
//
// Creation Date  :
//
//
//
//
//-------------------------------------------------------------------------

#include <Xyce_config.h>

#include <N_ERH_ErrorMgr.h>
#include <N_IO_OutputMgr.h>
#include <N_IO_OutputterFrequencyCSV.h>
#include <N_IO_OutputterFrequencyPrn.h>
#include <N_IO_OutputterTimeCSV.h>
#include <N_IO_OutputterTimePrn.h>
#include <N_IO_OutputterTimeProbe.h>
#include <N_IO_OutputterTimeRaw.h>
#include <N_IO_OutputterTimeRawASCII.h>
#include <N_IO_OutputterTimeTecplot.h>
#include <N_IO_OutputterTransient.h>

namespace Xyce {
namespace IO {
namespace Outputter {

//-----------------------------------------------------------------------------
// Function      : enableTransientOutput
// Purpose       :
// Special Notes :
// Scope         :
// Creator       : Dave Baur
// Creation Date : June 23, 2014
//-----------------------------------------------------------------------------
void enableTransientOutput(Parallel::Machine comm, OutputMgr &output_manager, Analysis::Mode analysis_mode)
{
  std::pair<OutputParameterMap::const_iterator, bool> result = output_manager.findOutputParameter(OutputType::TRAN);
  if (result.second)
  {
    for (std::vector<PrintParameters>::const_iterator it = (*result.first).second.begin(), end = (*result.first).second.end(); it != end; ++it) {
      PrintParameters transient_print_parameters = (*it);

        if (transient_print_parameters.format_ != Format::PROBE)
          transient_print_parameters.variableList_.push_front(Util::Param("TIME", 0.0));
        if (transient_print_parameters.printIndexColumn_)
          transient_print_parameters.variableList_.push_front(Util::Param("INDEX", 0.0));

        output_manager.fixupPrintParameters(comm, transient_print_parameters);

        Outputter::Interface *outputter;
        if (transient_print_parameters.format_ == Format::STD) {
          outputter = new Outputter::TimePrn(comm, output_manager, transient_print_parameters);
        }
        else if (transient_print_parameters.format_ == Format::CSV) {
          outputter = new Outputter::TimeCSV(comm, output_manager, transient_print_parameters);
        }
        else if (transient_print_parameters.format_ == Format::TECPLOT) {
          outputter = new Outputter::TimeTecplot(comm, output_manager, transient_print_parameters);
        }
        else if (transient_print_parameters.format_ == Format::PROBE) {
          outputter = new Outputter::TimeProbe(comm, output_manager, transient_print_parameters);
        }
        else if (transient_print_parameters.format_ == Format::RAW) {
            outputter = new Outputter::TimeRaw(comm, output_manager, transient_print_parameters);
        }
        else if (transient_print_parameters.format_ == Format::RAW_ASCII) {
          outputter = new Outputter::TimeRawAscii(comm, output_manager, transient_print_parameters);
        }
        else
        {
          Report::UserWarning0() << "Transient output cannot be written in " << transient_print_parameters.format_ << " format, using standard format";

          outputter = new Outputter::TimePrn(comm, output_manager, transient_print_parameters);
        }

        output_manager.addOutputter(PrintType::TRAN, outputter);
    }
  }
}

} // namespace Outputter
} // namespace IO
} // namespace Xyce
