//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2017 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  :
//
// Creator        : Eric Keiter
//
// Creation Date  : 5/22/00
//
//-------------------------------------------------------------------------

#ifndef Xyce_N_UTL_IndexPair_h
#define Xyce_N_UTL_IndexPair_h

namespace Xyce {

//-----------------------------------------------------------------------------
// Class         : IndexPair
// Purpose       : contains a pair of indices.  Mostly used for matrix
//                 access.
//
// Special Notes :
// Creator       : Eric Keiter, SNL, Parallel Computational Sciences
// Creation Date : 5/23/00
//-----------------------------------------------------------------------------
class IndexPair
{
public:
  static const int INIT_SUPER_FLAG = 1;

  IndexPair()
     : row(0),
       col(0),
       superFlag(0)
  {}

  IndexPair(
    int rowTmp,
    int colTmp,
    int superTmp = -1)
     : row(rowTmp),
       col(colTmp),
       superFlag(superTmp)
  {}

  int row, col, superFlag;
};

typedef std::vector<IndexPair> IndexPairVector;

//-----------------------------------------------------------------------------
// Function      : operator==
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : Rob Hoekstra, SNL, Parallel Computational Sciences
// Creation Date : 3/16/00
//-----------------------------------------------------------------------------
inline bool operator==(const IndexPair & ip1, const IndexPair & ip2)
{
  return ((ip1.row == ip2.row) && (ip1.col == ip2.col));
}

} // namespace Xyce

#endif // Xyce_N_UTL_IndexPair_h
